@@ -11,7 +11,7 @@ use Exporter 'import';
 our $AUTHORITY = 'cpan:PERLANCAR'; # AUTHORITY
 our $DATE = '2023-11-02'; # DATE
 our $DIST = 'File-MoreUtil'; # DIST
-our $VERSION = '0.627'; # VERSION
+our $VERSION = '0.628'; # VERSION
 
 our @EXPORT_OK = qw(
                        file_exists
@@ -42,6 +42,8 @@ our @EXPORT_OK = qw(
                        get_dir_files
                        get_dir_dot_files
                        get_dir_non_dot_files
+                       get_dir_only_file
+                       get_dir_only_subdir
                );
 
 our %SPEC;
@@ -288,7 +290,7 @@ sub get_dir_files {
     my ($dir) = @_;
     $dir //= ".";
     opendir my($dh), $dir or die "Can't opendir $dir: $!";
-    my @res = grep { $_ ne '.' && $_ ne '..' && -f } readdir $dh;
+    my @res = grep { $_ ne '.' && $_ ne '..' && (-f "$dir/$_")} readdir $dh;
     closedir $dh; # we're so nice
     @res;
 }
@@ -297,7 +299,7 @@ sub get_dir_dot_files {
     my ($dir) = @_;
     $dir //= ".";
     opendir my($dh), $dir or die "Can't opendir $dir: $!";
-    my @res = grep { $_ ne '.' && $_ ne '..' && /\A\./ && -f } readdir $dh;
+    my @res = grep { $_ ne '.' && $_ ne '..' && /\A\./ && (-f "$dir/$_")} readdir $dh;
     closedir $dh; # we're so nice
     @res;
 }
@@ -306,7 +308,7 @@ sub get_dir_non_dot_files {
     my ($dir) = @_;
     $dir //= ".";
     opendir my($dh), $dir or die "Can't opendir $dir: $!";
-    my @res = grep { $_ ne '.' && $_ ne '..' && !/\A\./ && -f } readdir $dh;
+    my @res = grep { $_ ne '.' && $_ ne '..' && !/\A\./ && (-f "$dir/$_")} readdir $dh;
     closedir $dh; # we're so nice
     @res;
 }
@@ -315,7 +317,7 @@ sub get_dir_subdirs {
     my ($dir) = @_;
     $dir //= ".";
     opendir my($dh), $dir or die "Can't opendir $dir: $!";
-    my @res = grep { $_ ne '.' && $_ ne '..' && !(-l) && (-d _) } readdir $dh;
+    my @res = grep { $_ ne '.' && $_ ne '..' && !(-l "$dir/$_") && (-d _) } readdir $dh;
     closedir $dh; # we're so nice
     @res;
 }
@@ -324,7 +326,7 @@ sub get_dir_non_subdirs {
     my ($dir) = @_;
     $dir //= ".";
     opendir my($dh), $dir or die "Can't opendir $dir: $!";
-    my @res = grep { $_ ne '.' && $_ ne '..' && ((-l) || !(-d _)) } readdir $dh;
+    my @res = grep { $_ ne '.' && $_ ne '..' && ((-l "$dir/$_") || !(-d _)) } readdir $dh;
     closedir $dh; # we're so nice
     @res;
 }
@@ -333,7 +335,7 @@ sub get_dir_dot_subdirs {
     my ($dir) = @_;
     $dir //= ".";
     opendir my($dh), $dir or die "Can't opendir $dir: $!";
-    my @res = grep { $_ ne '.' && $_ ne '..' && /\A\./ && !(-l) && (-d _) } readdir $dh;
+    my @res = grep { $_ ne '.' && $_ ne '..' && /\A\./ && !(-l "$dir/$_") && (-d _) } readdir $dh;
     closedir $dh; # we're so nice
     @res;
 }
@@ -342,11 +344,41 @@ sub get_dir_non_dot_subdirs {
     my ($dir) = @_;
     $dir //= ".";
     opendir my($dh), $dir or die "Can't opendir $dir: $!";
-    my @res = grep { $_ ne '.' && $_ ne '..' && !/\A\./ && !(-l) && (-d _) } readdir $dh;
+    my @res = grep { $_ ne '.' && $_ ne '..' && !/\A\./ && !(-l "$dir/$_") && (-d _) } readdir $dh;
     closedir $dh; # we're so nice
     @res;
 }
 
+sub get_dir_only_file {
+    my ($dir) = @_;
+    $dir //= ".";
+    opendir my($dh), $dir or die "Can't opendir $dir: $!";
+    my $res;
+    while (defined(my $e = readdir $dh)) {
+        next if $e eq '.' || $e eq '..';
+        return unless -f "$dir/$e";
+        return if defined $res;
+        $res = $e;
+    }
+    return unless defined $res;
+    $res;
+}
+
+sub get_dir_only_subdir {
+    my ($dir) = @_;
+    $dir //= ".";
+    opendir my($dh), $dir or die "Can't opendir $dir: $!";
+    my $res;
+    while (defined(my $e = readdir $dh)) {
+        next if $e eq '.' || $e eq '..';
+        return unless -d "$dir/$e";
+        return if defined $res;
+        $res = $e;
+    }
+    return unless defined $res;
+    $res;
+}
+
 1;
 # ABSTRACT: File-related utilities
 
@@ -362,7 +394,7 @@ File::MoreUtil - File-related utilities
 
 =head1 VERSION
 
-This document describes version 0.627 of File::MoreUtil (from Perl distribution File-MoreUtil), released on 2023-11-02.
+This document describes version 0.628 of File::MoreUtil (from Perl distribution File-MoreUtil), released on 2023-11-02.
 
 =head1 SYNOPSIS
 
@@ -393,6 +425,8 @@ This document describes version 0.627 of File::MoreUtil (from Perl distribution
      get_dir_files
      get_dir_dot_files
      get_dir_non_dot_files
+     get_dir_only_file
+     get_dir_only_subdir
  );
 
  print "file exists" if file_exists("/path/to/file/or/dir");
@@ -741,6 +775,24 @@ Basically a shortcut for something like:
 
  my @non_dot_subdirnames = do { opendir my $dh, $dir; grep { !/\A\./ && -d } readdir $dh };
 
+=head2 get_dir_only_file
+
+Usage:
+
+ my $filename = get_dir_only_file([ $dir ]);
+
+Return filename inside directory C<$dir> (or current directory if unspecified)
+only if C<$dir> has a single plain file and nothing else.
+
+=head2 get_dir_only_subdir
+
+Usage:
+
+ my $subdirname = get_dir_only_subdir([ $dir ]);
+
+Return subdirectory name inside directory C<$dir> (or current directory if
+unspecified) only if C<$dir> has a single subdirectory and nothing else.
+
 =head1 FAQ
 
 =head2 Where is file_empty()?
